"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeImportPath = exports.addScreenToAdminUi = void 0;
const j = __importStar(require("jscodeshift"));
const change_case_1 = require("change-case");
const path_1 = __importDefault(require("path"));
const recordRepresentationFunctionName_1 = require("../../../../../generators/react-admin/common/recordRepresentation/recordRepresentationFunctionName");
const insertSpecifierImport_1 = require("../jsCodeShift/insertSpecifierImport");
const parseTs_1 = require("../../../../util/parseTs");
function addScreenToAdminUi(appTsxInput, resourceName, screenName, screenType, screenImportPath) {
    // custom TS parser required
    const appAST = parseTs_1.parseTs(appTsxInput);
    const adminUi = appAST.findJSXElements("AdminUI");
    if (adminUi == null) {
        console.error(`addScreenToAdminUi: no 'AdminUI' tag found in input\n`, appTsxInput);
        throw new Error(`No 'AdminUI' tag found in input`);
    }
    if (adminUi.paths().length != 1) {
        console.error(`addScreenToAdminUi: menu.paths() should contains only one element`);
        throw new Error(`Incorrect data in 'menu.paths()'`);
    }
    const children = adminUi.paths()[0].value.children;
    if (children == null) {
        console.error(`addScreenToAdminUi: 'children' is undefined in input\n`, appTsxInput);
        throw new Error(`Parsing 'AdminUI' tag failed - 'children' is undefined`);
    }
    // try to find already existed <Resource name={resourceName}> tag in AdminUI
    const resourceItem = children
        .filter(el => el.type === j.JSXElement.toString())
        .find(el => {
        const openingEl = el.openingElement;
        return openingEl?.name?.name === "Resource" &&
            openingEl?.attributes?.some((attr) => attr.name?.name === "name" && attr.value?.value === resourceName);
    });
    if (resourceItem != null) {
        console.log(`addScreenToAdminUi: skip adding resource item to AdminUI tag, item ${resourceName} already exist`);
        const openingEl = resourceItem.openingElement;
        // try to find attribute, named as screenType in Resource tag
        const screenAttr = openingEl?.attributes?.find((attr) => attr.name.name === screenType);
        if (screenAttr != null) {
            // screen with such type already exist in Resource - overwrite attr value
            screenAttr.value = j.jsxExpressionContainer(j.jsxIdentifier(screenName));
            console.log(`addScreenToAdminUi: update attribute value to ${screenType}={ ${screenName} }'` +
                ` in ${resourceName} resource tag`);
        }
        else {
            // attribute for such screen type doesn't exist in Resource tag
            // add attribute `screenType={screenName}` to Resource tag
            // result example `<Resource create={ PetCreateAdvanced } />`
            const newScreenAttr = j.jsxAttribute(j.jsxIdentifier(screenType), j.jsxExpressionContainer(j.jsxIdentifier(screenName)));
            openingEl.attributes.push(newScreenAttr);
            console.log(`addScreenToAdminUi: attribute '${screenType}={ ${screenName} }' ` +
                `added to ${resourceName} resource tag`);
        }
    }
    else {
        createResourceTag(resourceName, screenType, screenName, children);
    }
    // find 'react-admin' import declaration
    insertSpecifierImport_1.insertSpecifierImport(appAST, 'Resource', 'react-admin');
    const importPath = normalizeImportPath(path_1.default.join(screenImportPath, screenName));
    insertSpecifierImport_1.insertSpecifierImport(appAST, screenName, importPath);
    const recordRepFunctionName = recordRepresentationFunctionName_1.recordRepresentationFunctionName(resourceName);
    insertSpecifierImport_1.insertSpecifierImport(appAST, recordRepFunctionName, recordRepresentationImportPath(recordRepFunctionName));
    return appAST.toSource();
}
exports.addScreenToAdminUi = addScreenToAdminUi;
function createResourceTag(resourceName, screenType, screenName, 
// TODO remove children from args
children) {
    const label = change_case_1.capitalCase(resourceName);
    const resourceItemTag = `` +
        `<Resource\n` +
        `  name="${resourceName}"\n` +
        `  options={{ label: "${label}" }}\n` +
        `  ${screenType}={${screenName}}\n` +
        `  recordRepresentation={${recordRepresentationFunctionName_1.recordRepresentationFunctionName(resourceName)}} />`;
    children?.push(j.stringLiteral(resourceItemTag));
    console.log(`addScreenToAdminUi: adding item to AdminUI tag \n${resourceItemTag} `);
}
// TODO move to common
// all imports are relative and should be of type `./path/to/screen/Screen`
function normalizeImportPath(importPath) {
    return importPath.startsWith(".") ? importPath : "./" + importPath;
}
exports.normalizeImportPath = normalizeImportPath;
function recordRepresentationImportPath(functionName) {
    return `../core/record-representation/${functionName}`;
}
//# sourceMappingURL=addScreenToAdminUi.js.map