"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCustomRouteToAdminUi = void 0;
const j = __importStar(require("jscodeshift"));
const path_1 = __importDefault(require("path"));
const insertSpecifierImport_1 = require("../jsCodeShift/insertSpecifierImport");
const parseTs_1 = require("../../../../util/parseTs");
const addScreenToAdminUi_1 = require("../resource/addScreenToAdminUi");
/**
 * Add Route with component to app. If Route with the same component name in Route path prop already exist - do nothing.
 * Example of added line `<Route path="/BlankComponent" element={<BlankComponent />} />`;
 *
 * @param appTsxInput text version of the file which will be changed
 * @param componentName name of added component
 * @param componentImportPath dir path where placed component
 * @param route routing path to component
 * @returns changed version of the file
 */
function addCustomRouteToAdminUi(appTsxInput, componentName, componentImportPath, route) {
    const appAST = parseTs_1.parseTs(appTsxInput);
    const adminUi = appAST.findJSXElements("AdminUI");
    if (adminUi == null) {
        console.error(`addScreenToAdminUi: no 'AdminUI' tag found in input\n`, appTsxInput);
        throw new Error(`No 'AdminUI' tag found in input`);
    }
    if (adminUi.paths().length != 1) {
        console.error(`addScreenToAdminUi: menu.paths() should contains only one element`);
        throw new Error(`Incorrect data in 'menu.paths()'`);
    }
    const children = adminUi.nodes()[0].children;
    if (children == null) {
        console.error(`addScreenToAdminUi: 'children' is undefined in input\n`, appTsxInput);
        throw new Error(`Parsing 'AdminUI' tag failed - 'children' is undefined`);
    }
    // try to find CustomRoutes tag
    let customRoutes = children
        .find(el => j.JSXElement.check(el)
        && j.JSXIdentifier.check(el.openingElement.name)
        && el.openingElement.name.name === 'CustomRoutes');
    // add CustomRoutes tag if not found
    if (customRoutes == null) {
        customRoutes = j.jsxElement(j.jsxOpeningElement(j.jsxIdentifier("CustomRoutes")), j.jsxClosingElement(j.jsxIdentifier("CustomRoutes")));
    }
    children.push(customRoutes);
    // try to find already existed <Route path="/BlankComponent" element={<BlankComponent />} /> tag in AdminUI
    const routeItem = customRoutes.children
        ?.filter(el => el.type === j.JSXElement.toString())
        .find(el => {
        if (!j.JSXElement.check(el) || !j.JSXIdentifier.check(el.openingElement.name)) {
            return false;
        }
        return el.openingElement.name.name === "Route"
            && el.openingElement.attributes?.some(attr => j.JSXAttribute.check(attr)
                && attr.name.name === "element"
                && j.JSXExpressionContainer.check(attr.value)
                && j.JSXElement.check(attr.value.expression)
                && j.JSXIdentifier.check(attr.value.expression.openingElement.name)
                && attr.value.expression.openingElement.name.name === componentName);
    });
    if (routeItem == null) {
        insertRouteTag(componentName, route, customRoutes.children);
        console.log(`addCustomRouteToAdminUi: adding route '${route}' to AdminUI tag`);
    }
    else {
        console.log(`addCustomRouteToAdminUi: skip adding Route to app, Route ${componentName} already exist`);
    }
    const importPath = addScreenToAdminUi_1.normalizeImportPath(path_1.default.join(componentImportPath, componentName));
    insertSpecifierImport_1.insertSpecifierImport(appAST, componentName, importPath);
    insertSpecifierImport_1.insertSpecifierImport(appAST, 'Route', 'react-router');
    insertSpecifierImport_1.insertSpecifierImport(appAST, 'CustomRoutes', 'react-admin');
    return appAST.toSource();
}
exports.addCustomRouteToAdminUi = addCustomRouteToAdminUi;
function insertRouteTag(componentName, route, children) {
    if (children == null)
        return;
    const routeTag = `<Route path="${route}" element={<${componentName} />} />`;
    const routeTagNode = parseTs_1.parseTs(routeTag).findJSXElements().nodes()[0];
    children.push(routeTagNode);
}
//# sourceMappingURL=addCustomRouteToAdminUi.js.map