"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addMenuItemToMenu = void 0;
const jscodeshift_1 = __importDefault(require("jscodeshift"));
const insertSpecifierImport_1 = require("../jsCodeShift/insertSpecifierImport");
/**
 * Add component to menu. If MenuItem with the same componentName in the primaryText prop already exist - do nothing.
 * Example of added line `<Menu.Item  to="/BlankComponent" primaryText="BlankComponent" />`;
 *
 * @param menuInput jsx menu as test
 * @param route routing path to component
 * @param componentName name of component to be added in menu
 */
function addMenuItemToMenu(menuInput, route, componentName) {
    const tsxParser = jscodeshift_1.default.withParser("tsx");
    const appMenuAST = tsxParser(menuInput);
    const menu = appMenuAST.findJSXElements("Menu");
    if (menu == null) {
        console.error(`addMenuItemToMenu: no 'Menu' tag found in input\n`, menuInput);
        throw new Error(`No 'Menu' tag found in input`);
    }
    if (menu.paths().length != 1) {
        console.error(`addMenuItemToMenu: menu.paths() should contains only one element`);
        throw new Error(`Incorrect data in 'menu.paths()'`);
    }
    const children = menu.nodes()[0].children;
    if (children == null) {
        console.error(`addMenuItemToMenu: 'children' is undefined in input\n`, menuInput);
        throw new Error(`Parsing 'Menu' tag failed - 'children' is undefined`);
    }
    // check, if such MenuItem item already exist in menu
    const isItemAlreadyExist = children
        .filter(el => el.type === jscodeshift_1.default.JSXElement.toString())
        .some(el => {
        const openingEl = el.openingElement;
        const openingElName = openingEl?.name;
        // looking for MenuItem like `<Menu.Item  to="/BlankComponent" primaryText="BlankComponent" />`,
        // where BlankComponent is componentName
        return openingElName?.object?.name === "Menu" &&
            openingElName?.property?.name === "Item" &&
            openingEl?.attributes?.some((attr) => attr.name.name === "primaryText" && attr.value.value === componentName);
    });
    if (!isItemAlreadyExist) {
        const menuItemTag = `<Menu.Item  to="${route}" primaryText="routes.${componentName}" leftIcon={<WebAsset />} />`;
        const menuItemTagNode = jscodeshift_1.default(menuItemTag).findJSXElements().nodes()[0];
        children.push(menuItemTagNode);
        insertSpecifierImport_1.insertSpecifierImport(appMenuAST, 'WebAsset', '@mui/icons-material/WebAsset', true);
        console.log(`MenuItem added to menu\n ${menuItemTag}`);
    }
    else {
        console.log(`skip adding MenuItem to menu, MenuItem ${componentName} already exist`);
    }
    return appMenuAST.toSource();
}
exports.addMenuItemToMenu = addMenuItemToMenu;
//# sourceMappingURL=addMenuItemToMenu.js.map